function f = consommateur(A,a,b,p1,p2,R)

%Fonction permettant de représenter les problème du consommateur, avec
%une fonction d'utilité de type Cobb-Douglas, de paramètres A, alpha et
%beta, et une contrainte de revenu du type p1*x1+p2*x2=R
%
%Nécessite de pouvoir afficher des objets graphiques
%
%Syntaxe : consommateur(A,alpha,beta,p1,p2,R);
%La fonction renvoie 1 si elle s'est exécutée correctement

steps=250; %Donne le degré de détails, plus la valeur est élevée, plus Matlab va générer d'échantillons pour représenter la courbe.

[opt x1 x2] = opti_conso(A,a,b,p1,p2,R); %On calcule les coordonnées de l'optimum.

conso = @(x,y) util_conso(x,y,A,a,b); %On crée une fonction qui pour tout point (x,y) et les paramètres du problème nous donne l'utilité

plot3(x1,x2,opt,'MarkerSize',10,'Marker','*'); %On place l'optimum sur la figure.

grid on; %On affiche le quadrillage
hold on; %On dit à Matlab d'afficher les prochaines figures sur le même dessin

xm = R/p1; %Maximum que peut prendre x1
ym = R/p2; %Idem pour x2
[xg yg] = meshgrid(0:xm/steps:xm,0:ym/steps:ym); %On génère un échantillonage dans l'intervalle 0<=x1<x1m, 0<=x2<=x2m

zg = conso(xg,yg); %On calcule toutes les utilités pour chaque couple de point (x1,x2) généré

[k l] = size(xg); %On récupère les dimensions de xg
for i=1:k %Parcours du tableau par ligne
    for j=1:l %Parcours par colonne
        if p1*xg(i,j)+p2*yg(i,j)>R %Est-ce que l'on viole la contrainte de revenu?
        zg(i,j) = NaN; %Si oui, alors on efface ce point
        end
    end
end

surf(xg,yg,zg,'FaceColor','interp','EdgeColor','none','FaceLighting','gouraud'); %On affiche la surface de la fonction d'utilité
% On n'y affiche pas les lignes, et on interpole les couleurs pour l'élévation

set(gca,{'XLim' 'YLim' 'ZLim'},{[0 xm] [0 ym] [0 opt+1]}); %On redimensionne la figure

plan = sqrt(xm^2+ym^2); %Longueur du plan représentant la contrainte de revenu
[yp zp] = meshgrid(0:plan/steps:plan,0:opt/steps:opt); %On crée un plan vertical de dimension plan*opt

alpha=atan(-p2/p1); %Angle de la rotation
yp2=yp*cos(alpha); %Rotation
xp=yp*sin(alpha)+xm; %Rotation+translation

[k l] = size(zp); %Cf précédemment, on va parcourir le tableau
for i=1:k
    for j=1:l
        if zp(i,j)>util_conso(xp(i,j),yp2(i,j),A,a,b) %Est-ce que mon plan se trouve au dessus de la surface représentant la fonction d'utilité?
        zp(i,j) = NaN; %Si oui, on l'enlève
        end
    end
end

surf(xp,yp2,zp,'FaceColor','interp','EdgeColor','none','FaceLighting','gouraud'); %On affiche le plan
xlabel('x1'); %On nomme l'axe des x1
ylabel('x2'); %Idem avec x2
zlabel('Utilité'); %On nomme l'ordonnée
text(x1,x2,opt+1,'Optimum'); % On met une étiquette pour signaler l'optimum
titlestr=sprintf('Problème du consommateur\nAvec A=%g, \\alpha=%g, \\beta=%g, p_1=%g, p_2=%g et R=%g',[A a b p1 p2 R]); %On crée une ligne de texte au format LaTeX pour titrer la figure
title(titlestr); %On affiche le titre

hold off; %Plus besoin de forcer les figures suivantes dans la même fenêtre

f=1; %L'exécution s'est bien déroulée
